# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/CFG.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/CFG.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_options=_b('\n\032com.grammatech.gtirb.proto'),
  serialized_pb=_b('\n\x15gtirb/proto/CFG.proto\x12\x0bgtirb.proto\"U\n\tEdgeLabel\x12\x13\n\x0b\x63onditional\x18\x01 \x01(\x08\x12\x0e\n\x06\x64irect\x18\x02 \x01(\x08\x12#\n\x04type\x18\x03 \x01(\x0e\x32\x15.gtirb.proto.EdgeType\"u\n\x04\x45\x64ge\x12\x13\n\x0bsource_uuid\x18\x01 \x01(\x0c\x12\x13\n\x0btarget_uuid\x18\x02 \x01(\x0c\x12%\n\x05label\x18\x05 \x01(\x0b\x32\x16.gtirb.proto.EdgeLabelJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\x07\x62ooleanR\x07integer\"G\n\x03\x43\x46G\x12\x10\n\x08vertices\x18\x03 \x03(\x0c\x12 \n\x05\x65\x64ges\x18\x02 \x03(\x0b\x32\x11.gtirb.proto.EdgeJ\x04\x08\x01\x10\x02R\x06\x62locks*t\n\x08\x45\x64geType\x12\x0f\n\x0bType_Branch\x10\x00\x12\r\n\tType_Call\x10\x01\x12\x14\n\x10Type_Fallthrough\x10\x02\x12\x0f\n\x0bType_Return\x10\x03\x12\x10\n\x0cType_Syscall\x10\x04\x12\x0f\n\x0bType_Sysret\x10\x05\x42\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
)

_EDGETYPE = _descriptor.EnumDescriptor(
  name='EdgeType',
  full_name='gtirb.proto.EdgeType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Type_Branch', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Type_Call', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Type_Fallthrough', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Type_Return', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Type_Syscall', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Type_Sysret', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=317,
  serialized_end=433,
)
_sym_db.RegisterEnumDescriptor(_EDGETYPE)

EdgeType = enum_type_wrapper.EnumTypeWrapper(_EDGETYPE)
Type_Branch = 0
Type_Call = 1
Type_Fallthrough = 2
Type_Return = 3
Type_Syscall = 4
Type_Sysret = 5



_EDGELABEL = _descriptor.Descriptor(
  name='EdgeLabel',
  full_name='gtirb.proto.EdgeLabel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conditional', full_name='gtirb.proto.EdgeLabel.conditional', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='direct', full_name='gtirb.proto.EdgeLabel.direct', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='gtirb.proto.EdgeLabel.type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=123,
)


_EDGE = _descriptor.Descriptor(
  name='Edge',
  full_name='gtirb.proto.Edge',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source_uuid', full_name='gtirb.proto.Edge.source_uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='target_uuid', full_name='gtirb.proto.Edge.target_uuid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='gtirb.proto.Edge.label', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=242,
)


_CFG = _descriptor.Descriptor(
  name='CFG',
  full_name='gtirb.proto.CFG',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vertices', full_name='gtirb.proto.CFG.vertices', index=0,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='edges', full_name='gtirb.proto.CFG.edges', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=244,
  serialized_end=315,
)

_EDGELABEL.fields_by_name['type'].enum_type = _EDGETYPE
_EDGE.fields_by_name['label'].message_type = _EDGELABEL
_CFG.fields_by_name['edges'].message_type = _EDGE
DESCRIPTOR.message_types_by_name['EdgeLabel'] = _EDGELABEL
DESCRIPTOR.message_types_by_name['Edge'] = _EDGE
DESCRIPTOR.message_types_by_name['CFG'] = _CFG
DESCRIPTOR.enum_types_by_name['EdgeType'] = _EDGETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EdgeLabel = _reflection.GeneratedProtocolMessageType('EdgeLabel', (_message.Message,), {
  'DESCRIPTOR' : _EDGELABEL,
  '__module__' : 'gtirb.proto.CFG_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.EdgeLabel)
  })
_sym_db.RegisterMessage(EdgeLabel)

Edge = _reflection.GeneratedProtocolMessageType('Edge', (_message.Message,), {
  'DESCRIPTOR' : _EDGE,
  '__module__' : 'gtirb.proto.CFG_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.Edge)
  })
_sym_db.RegisterMessage(Edge)

CFG = _reflection.GeneratedProtocolMessageType('CFG', (_message.Message,), {
  'DESCRIPTOR' : _CFG,
  '__module__' : 'gtirb.proto.CFG_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.CFG)
  })
_sym_db.RegisterMessage(CFG)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
