# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/ByteInterval.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gtirb.proto import CodeBlock_pb2 as gtirb_dot_proto_dot_CodeBlock__pb2
from gtirb.proto import DataBlock_pb2 as gtirb_dot_proto_dot_DataBlock__pb2
from gtirb.proto import SymbolicExpression_pb2 as gtirb_dot_proto_dot_SymbolicExpression__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/ByteInterval.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_options=_b('\n\032com.grammatech.gtirb.proto'),
  serialized_pb=_b('\n\x1egtirb/proto/ByteInterval.proto\x12\x0bgtirb.proto\x1a\x1bgtirb/proto/CodeBlock.proto\x1a\x1bgtirb/proto/DataBlock.proto\x1a$gtirb/proto/SymbolicExpression.proto\"p\n\x05\x42lock\x12\x0e\n\x06offset\x18\x01 \x01(\x04\x12&\n\x04\x63ode\x18\x02 \x01(\x0b\x32\x16.gtirb.proto.CodeBlockH\x00\x12&\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x16.gtirb.proto.DataBlockH\x00\x42\x07\n\x05value\"\xb5\x02\n\x0c\x42yteInterval\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\"\n\x06\x62locks\x18\x02 \x03(\x0b\x32\x12.gtirb.proto.Block\x12P\n\x14symbolic_expressions\x18\x03 \x03(\x0b\x32\x32.gtirb.proto.ByteInterval.SymbolicExpressionsEntry\x12\x13\n\x0bhas_address\x18\x04 \x01(\x08\x12\x0f\n\x07\x61\x64\x64ress\x18\x05 \x01(\x04\x12\x0c\n\x04size\x18\x06 \x01(\x04\x12\x10\n\x08\x63ontents\x18\x07 \x01(\x0c\x1a[\n\x18SymbolicExpressionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x04\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.gtirb.proto.SymbolicExpression:\x02\x38\x01\x42\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
  ,
  dependencies=[gtirb_dot_proto_dot_CodeBlock__pb2.DESCRIPTOR,gtirb_dot_proto_dot_DataBlock__pb2.DESCRIPTOR,gtirb_dot_proto_dot_SymbolicExpression__pb2.DESCRIPTOR,])




_BLOCK = _descriptor.Descriptor(
  name='Block',
  full_name='gtirb.proto.Block',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.Block.offset', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='code', full_name='gtirb.proto.Block.code', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='gtirb.proto.Block.data', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='gtirb.proto.Block.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=143,
  serialized_end=255,
)


_BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY = _descriptor.Descriptor(
  name='SymbolicExpressionsEntry',
  full_name='gtirb.proto.ByteInterval.SymbolicExpressionsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='gtirb.proto.ByteInterval.SymbolicExpressionsEntry.key', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='gtirb.proto.ByteInterval.SymbolicExpressionsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=476,
  serialized_end=567,
)

_BYTEINTERVAL = _descriptor.Descriptor(
  name='ByteInterval',
  full_name='gtirb.proto.ByteInterval',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='gtirb.proto.ByteInterval.uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blocks', full_name='gtirb.proto.ByteInterval.blocks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbolic_expressions', full_name='gtirb.proto.ByteInterval.symbolic_expressions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='has_address', full_name='gtirb.proto.ByteInterval.has_address', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='address', full_name='gtirb.proto.ByteInterval.address', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='gtirb.proto.ByteInterval.size', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contents', full_name='gtirb.proto.ByteInterval.contents', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=258,
  serialized_end=567,
)

_BLOCK.fields_by_name['code'].message_type = gtirb_dot_proto_dot_CodeBlock__pb2._CODEBLOCK
_BLOCK.fields_by_name['data'].message_type = gtirb_dot_proto_dot_DataBlock__pb2._DATABLOCK
_BLOCK.oneofs_by_name['value'].fields.append(
  _BLOCK.fields_by_name['code'])
_BLOCK.fields_by_name['code'].containing_oneof = _BLOCK.oneofs_by_name['value']
_BLOCK.oneofs_by_name['value'].fields.append(
  _BLOCK.fields_by_name['data'])
_BLOCK.fields_by_name['data'].containing_oneof = _BLOCK.oneofs_by_name['value']
_BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY.fields_by_name['value'].message_type = gtirb_dot_proto_dot_SymbolicExpression__pb2._SYMBOLICEXPRESSION
_BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY.containing_type = _BYTEINTERVAL
_BYTEINTERVAL.fields_by_name['blocks'].message_type = _BLOCK
_BYTEINTERVAL.fields_by_name['symbolic_expressions'].message_type = _BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY
DESCRIPTOR.message_types_by_name['Block'] = _BLOCK
DESCRIPTOR.message_types_by_name['ByteInterval'] = _BYTEINTERVAL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Block = _reflection.GeneratedProtocolMessageType('Block', (_message.Message,), {
  'DESCRIPTOR' : _BLOCK,
  '__module__' : 'gtirb.proto.ByteInterval_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.Block)
  })
_sym_db.RegisterMessage(Block)

ByteInterval = _reflection.GeneratedProtocolMessageType('ByteInterval', (_message.Message,), {

  'SymbolicExpressionsEntry' : _reflection.GeneratedProtocolMessageType('SymbolicExpressionsEntry', (_message.Message,), {
    'DESCRIPTOR' : _BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY,
    '__module__' : 'gtirb.proto.ByteInterval_pb2'
    # @@protoc_insertion_point(class_scope:gtirb.proto.ByteInterval.SymbolicExpressionsEntry)
    })
  ,
  'DESCRIPTOR' : _BYTEINTERVAL,
  '__module__' : 'gtirb.proto.ByteInterval_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.ByteInterval)
  })
_sym_db.RegisterMessage(ByteInterval)
_sym_db.RegisterMessage(ByteInterval.SymbolicExpressionsEntry)


DESCRIPTOR._options = None
_BYTEINTERVAL_SYMBOLICEXPRESSIONSENTRY._options = None
# @@protoc_insertion_point(module_scope)
