# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/Symbol.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/Symbol.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_options=_b('\n\032com.grammatech.gtirb.proto'),
  serialized_pb=_b('\n\x18gtirb/proto/Symbol.proto\x12\x0bgtirb.proto\"\x86\x01\n\x06Symbol\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x0f\n\x05value\x18\x02 \x01(\x04H\x00\x12\x17\n\rreferent_uuid\x18\x05 \x01(\x0cH\x00\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0e\n\x06\x61t_end\x18\x06 \x01(\x08\x42\x12\n\x10optional_payloadJ\x04\x08\x04\x10\x05R\x0cstorage_kindB\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
)




_SYMBOL = _descriptor.Descriptor(
  name='Symbol',
  full_name='gtirb.proto.Symbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='gtirb.proto.Symbol.uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='gtirb.proto.Symbol.value', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='referent_uuid', full_name='gtirb.proto.Symbol.referent_uuid', index=2,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='gtirb.proto.Symbol.name', index=3,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='at_end', full_name='gtirb.proto.Symbol.at_end', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='optional_payload', full_name='gtirb.proto.Symbol.optional_payload',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=42,
  serialized_end=176,
)

_SYMBOL.oneofs_by_name['optional_payload'].fields.append(
  _SYMBOL.fields_by_name['value'])
_SYMBOL.fields_by_name['value'].containing_oneof = _SYMBOL.oneofs_by_name['optional_payload']
_SYMBOL.oneofs_by_name['optional_payload'].fields.append(
  _SYMBOL.fields_by_name['referent_uuid'])
_SYMBOL.fields_by_name['referent_uuid'].containing_oneof = _SYMBOL.oneofs_by_name['optional_payload']
DESCRIPTOR.message_types_by_name['Symbol'] = _SYMBOL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Symbol = _reflection.GeneratedProtocolMessageType('Symbol', (_message.Message,), {
  'DESCRIPTOR' : _SYMBOL,
  '__module__' : 'gtirb.proto.Symbol_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.Symbol)
  })
_sym_db.RegisterMessage(Symbol)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
